# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import socket
import sys
import urllib

class FileDownload:

	def __init__(self, url = None, file=None, report_handler=None):
		self.url = url
		self.file = file
		self.report_handler = report_handler

	def download_file(self, url=None, file=None, report_handler=None):
		if url != None:
			self.url = url
			
		if file != None:
			self.file = file
			
		if report_handler != None:
			self.report_handler = None
			
		if self.url == None or self.file == None:
			raise Exception, "url and file are required arguments"


		old_socket_timeout = socket.getdefaulttimeout()
		socket.setdefaulttimeout(10)
		
		try:
			urllib.urlretrieve(url, file, report_handler)
			urllib.urlcleanup()
		except IOError, e:
			raise e
		
		file = None
		
		socket.setdefaulttimeout(old_socket_timeout)
		
	def destroy(self):
		self = None




# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import random
import pySocket
import sys

class DynamicPorts:
	def __randomize(self):
		return random.randrange(3000, 65535, 1)
	
	def __check_available(self, port):
		s = pySocket.pySocket("127.0.0.1")
		s.set_port(port)
		return s.is_available()
	
	def __get_available_port(self):
		port = self.__randomize()
		while self.__check_available(port) == False:
			port = self.__randomize()
		
		return port
		
	def get_ports(self):
		inbound_port = self.__get_available_port()
		outbound_port = self.__get_available_port()
		
		return inbound_port, outbound_port
		
if __name__ == '__main__':
	dyn = DynamicPorts()
	inbound_port, outbound_port = dyn.get_ports()
	print "in: %d out: %d" % (inbound_port, outbound_port)



# pySopCast implementation of ConfigurationManager
# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

from ConfigurationManager import *
import os
import locale

class ChannelGuideLanguages:
	ENGLISH = 0
	CHINESE = 1

class ChannelGuideLayout:
	UNITY = 0
	DUAL_WINDOW = 1
	
class ChannelGuideAutoRefresh:
	NEVER = 0
	DAILY = 1
	PROGRAM_START = 2

cur_locale = locale.setlocale(locale.LC_ALL, "")

def is_chinese():
	return not cur_locale[:len("zh".lower())] != "zh".lower()

class pySopCastConfigurationManager(ConfigurationManager):
	def __init__(self):
		ConfigurationManager.__init__(self, os.path.expanduser('~/.pySopCast/pySopCast.cfg'))
		
		if is_chinese() == True:
			language = _("Chinese")
			channel_guide = "http://channel.sopcast.com/gchlxml"
		else:
			language = _("English")
			channel_guide = "http://www.sopcast.com/gchlxml"
					     
		self.add_section("player", { "show_toolbar" : True,
					     "static_ports" : False,
					     "width" : 600,
					     "height" : 400,
					     "div_position" : 300,
					     "show_channel_guide" : True,
					     "inbound_port" : 8901,
					     "outbound_port" : 8902,
					     "volume" : 100,
					     "server" : "127.0.0.1",
					     "channel_guide_width" : 80,
					     "external_player" : False,
					     "external_player_command" : "mplayer -ontop -geometry 100%%:100%%",
					     "channel_timeout" : 3,
					     "stay_on_top" : False,
					     "show_channel_guide_toolbar_item" : True,
					     "new_bindings" : True,
					     "layout" : ChannelGuideLayout.UNITY })
						  
		self.add_section("ChannelGuide", {  "width" : 600,
						    "height" : 400,
						    "auto_refresh" : False,
						    "auto_refresh_frequency" : ChannelGuideAutoRefresh.NEVER,
						    "channel_guide_language" : language,
						    "last_updated" : "Never",
						    "url" : channel_guide,
						    "div_position" : 200 })
		self.read()
	
	def uses_new_bindings(self, value=None):
		return self.__getter_setter("getboolean", "player", "new_bindings", value)
	
	def player_width(self, value=None):
		return self.__getter_setter("getint", "player", "width", value)
	
	def player_height(self, value=None):
		return self.__getter_setter("getint", "player", "height", value)
		
	def inbound_static_port(self, value=None):
		return self.__getter_setter("getint", "player", "inbound_port", value)
	
	def outbound_static_port(self, value=None):
		return self.__getter_setter("getint", "player", "outbound_port", value)
		
	def server(self, value=None):
		return self.__getter_setter("get", "player", "server", value)
		
	def display_pane_position(self, value=None):
		return self.__getter_setter("getint", "player", "div_position", value)
		
	def player_volume(self, value=None):
		return self.__getter_setter("getint", "player", "volume", value)
	
	def show_channel_guide(self, value=None):
		return self.__getter_setter("getboolean", "player", "show_channel_guide", value)
		
	def use_static_ports(self, value=None):
		return self.__getter_setter("getboolean", "player", "static_ports", value)
		
	def stay_on_top(self, value=None):
		return self.__getter_setter("getboolean", "player", "stay_on_top", value)
		
	def use_external_player(self, value=None):
		return self.__getter_setter("getboolean", "player", "external_player", value)

	def external_player_command(self, value=None):
		return self.__getter_setter("get", "player", "external_player_command", value)
	
	def channel_guide_url(self, value=None):
		return self.__getter_setter("get", "ChannelGuide", "url", value)
	
	def channel_guide_language(self, value=None):
		return self.__getter_setter("get", "ChannelGuide", "channel_guide_language", value)

	def channel_guide_pane_position(self, value=None):
		return self.__getter_setter("getint", "ChannelGuide", "div_position", value)
		
	def channel_guide_width(self, value=None):
		return self.__getter_setter("getint", "ChannelGuide", "width", value)
	
	def channel_timeout(self, value=None):
		return self.__getter_setter("getint", "player", "channel_timeout", value)
	
	def __getter_setter(self, function, section, parameter, value=None):
		if value is None:
			return getattr(super(pySopCastConfigurationManager, self), function)(section, parameter)
		else:
			self.set(section, parameter, value)
			self.write()
			
			return
	



# Copyright (C) 2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import fork
import sys
import threading
import time
import os
import gtk
import locale
import gettext

cur_locale = locale.setlocale(locale.LC_ALL, "")

gtk.glade.bindtextdomain("sopcast-player")
gtk.glade.textdomain("sopcast-player")

gettext.bindtextdomain("sopcast-player")
gettext.textdomain("sopcast-player")

lang = gettext.translation("sopcast-player", languages=[cur_locale], fallback=True)
lang.install('sopcast-player')

class UpdateUIThread(threading.Thread):
	def __init__ (self, parent, channel_timeout=3):
		threading.Thread.__init__(self)
		self.daemon = True
		self.parent = parent
		self.sleep_time = .1
		self.terminate = False
		self.run_thread = False
		self.play_stream = False
		self.wait_before_retry_time = channel_timeout / self.sleep_time
		self.wait_before_restart = 3 / self.sleep_time
		self.time_waiting = 0
		self.paused = True
		self.terminated = False
		self.retry = False
		self.volume = None
		self.external_player = fork.ForkExternalPlayer()
		
	def run(self):
		err_point = 0
		i = 0
		loading = False
		retry = False
		was_playing = False
		while self.terminate == False:
			if self.run_thread == True:
				self.paused = False
				if self.parent.fork_sop.is_running() == True and self.run_thread == True:
					stats = self.parent.sop_stats.update_stats()
					
					if stats == None and self.run_thread == True:
						if was_playing == True and self.run_thread == True:
							if self.parent.external_player_command != None:
								if self.parent.external_player_command != "":
									self.external_player.kill()
									print("stoping external media player")
							else:
								self.parent.stop_vlc()						
								
							was_playing = False
							
						if retry == True and self.run_thread == True:
							self.time_waiting += 1
							
							if self.time_waiting > self.wait_before_restart and self.run_thread == True:
								self.parent.fork_sop.kill_sop()
							else:
								gtk.gdk.threads_enter()
								self.parent.update_statusbar("%s" % _("Retrying channel"))
								gtk.gdk.threads_leave()
						else:
							if loading == True and self.run_thread == True:
								self.time_waiting += 1
								if self.time_waiting > self.wait_before_retry_time and self.run_thread == True:
									retry = True
									self.time_waiting = 0
							loading = True
							
							gtk.gdk.threads_enter()
							self.parent.update_statusbar("%s" % _("Connecting"))
							gtk.gdk.threads_leave()
					
							if self.play_stream == True and self.run_thread == True:
								self.play_stream = False
					else:
						self.time_waiting = 0
						loading = False
						retry = False
						started = True
						
						if stats != None:
							if int(stats[0]) < 10 and self.run_thread == True:
								if i == 5 and self.run_thread == True:
									if int(stats[0]) > 0 and self.run_thread == True:
										gtk.gdk.threads_enter()
										self.parent.update_statusbar("%s: %s%%" % (_("Buffering"), stats[0]))
										gtk.gdk.threads_leave()
									else:
										gtk.gdk.threads_enter()
										self.parent.update_statusbar("%s" % _("Connecting"))
										gtk.gdk.threads_leave()
									i = 0
							else:
								if i == 5 and self.run_thread == True:
									gtk.gdk.threads_enter()
									self.parent.update_statusbar("%s: %s%%" % (_("Buffer"), stats[0]))
									gtk.gdk.threads_leave()
								
									i = 0
								
								if self.play_stream == False and self.run_thread == True:
									gtk.gdk.threads_enter()
									self.parent.update_statusbar("%s: %s%%" % (_("Buffer"), stats[0]))
									gtk.gdk.threads_leave()
									
									if self.parent.external_player_command != None:
										if self.parent.external_player_command != "":
											self.external_player.fork_player(self.parent.external_player_command, self.parent.outbound_media_url)
											print("Executing " + self.parent.external_player_command)
									else:
										gtk.gdk.threads_enter()
										started = self.parent.start_vlc()
										gtk.gdk.threads_leave()								
								
									if started == True and self.run_thread == True:
										self.play_stream = True
										was_playing = True
							
								loading = False
							i += 1

				else:
					gtk.gdk.threads_enter()
					self.parent.update_statusbar("")
					gtk.gdk.threads_leave()
					
					self.play_stream = False
					loading = False
					
					gtk.gdk.threads_enter()
					self.parent.play_channel()
					gtk.gdk.threads_leave()
					
					self.time_waiting = 0
					loading = False
					retry = False
			else:
				self.paused = True
									
			time.sleep(self.sleep_time)
		self.external_player.kill()
		self.terminated = True
		
	def print_point_on_exit(self, point):
		if self.terminate == True:
			print(point)
	
	def set_channel_timeout(self, channel_timeout):
		if channel_timeout != sys.maxint:
			self.wait_before_retry_time = channel_timeout / self.sleep_time
		else:
			self.wait_before_retry_time = sys.maxint
		
	def startup(self):
		self.run_thread = True
		
	def shutdown(self):
		self.run_thread = False
		self.play_stream = False
			
	def stop(self):
		self.shutdown()
		self.terminate = True
		
		while self.terminated == False:
			time.sleep(self.sleep_time)
		
		return True



# Copyright (C) 2009-2011 Jason Scheunemann <jason.scheunemann@yahoo.com>.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import socket

class pySocket:
	def __init__(self, host=None, port=None):
		self.host = host
		self.port = port
		
	def set_host(self, host):
		self.host = host
		
	def set_port(self, port):
		self.port = port
		
	def is_available(self, host=None, port=None):
		if host != None:
			self.host = host
			
		if port != None:
			self.port = port
		
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		return not self.sock.connect_ex((self.host, self.port)) == 0
